/* buscaPGM.pas
Programa que procura um padrao de imagem no formato PGM em uma imagem
maior e coloca uma borda preta sobre o padrao encontrado.

Complete o programa com:
1. a funcao achouPadrao (im, linIni, colIni, imP, numLinP, numColP):
  retorna 1 se o padrao de imagem imP com tamanho numLinP e numColP
  for encontrado na imagem im com o canto superior esquerdo na posicao
  (linIni, colIni)

2. procedimento bordaPreta (im, linIni, colIni, numLin, numCol)
  altera a imagem im, colocando uma borda de cor PRETO de tamanho
  numLin x numCol com o canto superior esquerdo na posicao (linIni, colIni).

Teste seu programa com os arquivos de imagem:
1. cenaWally.pgm e wally.pgm
2. circulos.pgm e padrao.pgm
*/

#include <iostream>
#include <fstream>
using namespace std;

#define MAX    1024                        /* dimensao maxima da imagem (MAX x MAX) */
#define LIN_MAX  60                        /* numLin maximo da imagem padrao (LIN_MAX x MAX) */
#define PRETO  0

void leArqPGM( istream &f,
	       int  m[][MAX], int &numLin, int &numCol, int &numMax );
void escreveArqPGM( ostream &f,
		    int m[][MAX], int numLin, int numCol, int numMax );
int achouPadrao(int im[][MAX], int linIni, int colIni,
		int imP[][MAX], int numLinP, int numColP);
void bordaPreta (int im[][MAX], int linIni, int colIni, int numLin, int numCol);

  
int main(){
  char nomeArqIm[50], nomeArqP[50], nomeArqSai[50];  /* nomes externos dos arquivos */
  int im[MAX][MAX], imP[LIN_MAX][MAX];               /* imagem e padrao */
  int numLin, numCol, numMax;              /* dimensao e valor do branco na imagem */
  int linP, colP, maxP;                    /* dimensao e valor do branco no padrao */
  int i, j;                                /* indices */

   /* leitura dos nomes dos arquivos de imagem e padrão e 
      abertura dos arquivos para leitura */
   cout << "Arquivo imagem PGM: ";  
   cin >> nomeArqIm;
   ifstream arqIm( nomeArqIm );
   if( !arqIm.is_open()){
     cout << "Erro na abertura do arquivo de imagem\n";
     return EXIT_FAILURE;
   }
   leArqPGM( arqIm, im, numLin, numCol, numMax );
   arqIm.close( );

   cout << "Arquivo padrao PGM: ";
   cin >> nomeArqP;
   ifstream arqP( nomeArqP );
   if( !arqP.is_open()){
     cout << "Erro na abertura do arquivo de padrao\n";
     return EXIT_FAILURE;
   }
   leArqPGM( arqP, imP, linP, colP, maxP );
   arqP.close();

   /* As posições da matriz da imagem entre as coordenadas
      [0,0] e [numLin-linP, numCol-colP] podem ser o canto 
      superior esquerdo de uma ocorrencia da imagem padrao */
   for( i=0; i<=numLin-linP; i++ )
     for( j=0; j<=numCol-colP; j++ )
       if( achouPadrao(im, i, j, imP, linP, colP))
	 bordaPreta(im, i, j, linP, colP);

   cout << "Arquivo saida PGM: ";
   cin >> nomeArqSai;
   ofstream arqSai( nomeArqSai );                        
   if( !arqSai.is_open()){
     cout << "Erro na abertura do arquivo de saida\n";
     return EXIT_FAILURE;
   }
   escreveArqPGM( arqSai, im, numLin, numCol, numMax );
   arqSai.close();
   return 0;
}
   
/* ----------------------------------------------------------------------
leArqPGM: le um arquivo PGM
Entrada: arquivo f que contem a imagem PGM
Saida:   m: matriz contendo os pixels da imagem
         numLin, numCol: dimensão da imagem
         numMax: valor que representa o branco
*/
void leArqPGM( istream &f,
	       int  m[][MAX], int &numLin, int &numCol, int &numMax ){
  string id;
  int i, j;

  f >> id;                         /* leitura do identificador "P2" */
  f >> numCol >> numLin;           /* leitura da dimensão da imagem */
  f >> numMax;                     /* leitura do inteiro que representa o branco */
  for( i=0; i<numLin; i++)         /* leitura da matriz */
    for( j=0; j<numCol; j++ )
      f >> m[i][j];
}

/* ----------------------------------------------------------------------
escreveArqPGM: escreve um arquivo PGM
Entrada: m: matriz contendo os pixels da imagem
         numLin, numCol: dimensão da imagem
         numMax: valor que representa o branco
Saida:   f: o arquivo no formato PGM
*/
void escreveArqPGM( ostream &f,
		    int m[][MAX], int numLin, int numCol, int numMax ){
  int i, j;

  f << "P2" << endl;
  f << numCol << " " << numLin << endl;
  f << numMax << endl;
  for( i=0; i<numLin; i++ ){
    for( j=0; j<numCol; j++ )
      f << m[i][j] << " ";
    f << endl;
  }
}


/*  ----------------------------------------------------------------------
Funcao achouPadrao:
Retorna 1 se encontrou a imagem padrao imP na imagem im com o canto
superior esquerdo em uma determinada posicao; retorna 0, caso contrario.
Entrada: im: matriz com a imagem onde é feita a busca
         linIni, colIni: coordenada do canto superior esquerdo a partir
                         de onde é feita a busca
         imP: matriz com o padrão a ser buscado
         numLinP, numColP: dimensão da matriz imP
*/

int achouPadrao(int im[][MAX], int linIni, int colIni,
		int imP[][MAX], int numLinP, int numColP){
}

/* ----------------------------------------------------------------------
Procedimento bordaPreta:
altera a imagem im, colocando uma borda de cor PRETO de tamanho
numLin x numCol com o canto superior esquerdo na posicao (linIni, colIni)
Entrada: im: matriz contendo a imagem
         linIni, colIni: coordenada do canto superior esquerdo
         numLin, numCol: dimensão do retângulo
Saida:   im: matriz com a imagem alterada 
 */

void bordaPreta (int im[][MAX], int linIni, int colIni, int numLin, int numCol){
  int  l, c, linFim, colFim;

}

